Summary: A collection of tools for managing UNIX services
Name: daemontools
Version: 0.76
Release: 3%{?dist}
License: See djb@pobox.com
Group: Utilities/System
Source0: http://cr.yp.to/daemontools/daemontools-%{version}.tar.gz
BuildRoot: /tmp/daemontools-buildroot
URL: http://cr.yp.to/daemontools.html
Packager: John Simpson <jms1@voalte.com>

%description
daemontools is a collection of tools for managing UNIX services. 

supervise monitors a service. It starts the service and restarts the
service if it dies. Setting up a new service is easy: all supervise
needs is a directory with a run script that runs the service.

multilog saves error messages to one or more logs. It optionally
timestamps each line and, for each log, includes or excludes lines
matching specified patterns. It automatically rotates logs to limit the
amount of disk space used. If the disk fills up, it pauses and tries
again, without losing any data.

%prep
%setup -n admin/daemontools-0.76/src

echo gcc %{optflags} -I. >conf-cc
echo gcc %{optflags} -I. -s >conf-ld
echo %{buildroot}/usr/local >home

mkdir sys
echo '#include </usr/include/sys/time.h>' >sys/time.h
echo '#include </usr/include/time.h>' >>sys/time.h

sed -e 's/extern int errno;/#include <errno.h>/' <error.h >error.h.tmp
mv error.h.tmp error.h

%build
make

%install
rm -fr %{buildroot}
mkdir -p %{buildroot}/usr/local/bin
while read command; do
	install -m 0755 $command %{buildroot}/usr/local/bin/$command
done <../package/commands

%clean
rm -rf %{buildroot}

%pre
###############################################################################
#
# pre-install script
#
# If this is an upgrade instead of a fresh install, we need to remove whatever
# init scripts were installed by the previous package, and stop the currently
# running services. We do NOT, however, want to totally remove the services.

PATH="/usr/bin:/bin:/usr/local/bin"

#echo "===== start %{version}-%{release} %pre $1 ====="
#set -x

if [[ "$1" == "2" ]]    # doing an upgrade, as opposed to a new install
then
    ########################################
    # if it's in systemd, stop the service and remove the unit file

    if [[ -f /lib/systemd/system/daemontools.service ]]
    then
        echo Found systemd unit file
        systemctl stop daemontools.service
        systemctl disable daemontools.service
        rm -f /lib/systemd/system/daemontools.service
    fi

    ########################################
    # if it's in upstart (either location),
    # stop it and remove the upstart config

    for DIR in /etc/init /etc/event.d
    do
        if [[ -f $DIR/daemontools.conf ]]
        then
            echo Found $DIR/daemontools.conf, removing from upstart
            /sbin/initctl stop daemontools
            rm -f $DIR/daemontools.conf
        fi
    done

    ########################################
    # if it's in inittab, remove it and send a HUP to init

    if [[ -f /etc/inittab ]]
    then
        if grep svscanboot /etc/inittab >/dev/null
        then
            echo 'Removing svscanboot from /etc/inittab'

            grep -v svscanboot /etc/inittab > /etc/inittab'{new}'
            chmod 0644 /etc/inittab'{new}'
            mv -f /etc/inittab'{new}' /etc/inittab
            kill -HUP 1
        fi
    fi

    ########################################
    # if it's in /etc/rc.local, remove it from there

    if [[ -f /etc/rc.local ]]
    then
        if grep svscanboot /etc/rc.local >/dev/null
        then
            echo 'Removing svscanboot from /etc/rc.local'

            grep -v svscanboot /etc/rc.local > /etc/rc.local'{new}'
            chmod 0644 /etc/rc.local'{new}'
            mv -f /etc/rc.local'{new}' /etc/rc.local
        fi
    fi

    ########################################
    # wherever it was, make sure it's dead

    KARG=""
    X=`ps -o pid= -C svscanboot`
    while [[ -n "$X" ]]
    do
        echo "Killing$KARG svscanboot [$X]"
        kill $KARG $X

        sleep 3
        KARG=" -9"
        X=`ps -o pid= -C svscanboot`
    done

    KARG=""
    X=`ps -o pid= -C svscan`
    while [[ -n "$X" ]]
    do
        echo "Killing$KARG svscan [$X]"
        kill $KARG $X

        sleep 3
        KARG=" -9"
        X=`ps -o pid= -C svscan`
    done

    ########################################
    # At this point, svscan is fully stopped, which means we can run `svc -dx`
    # on each service directory without having to worry about svscan starting
    # the services back up.

    svc -dx /service/* /service/*/log 2>/dev/null

    ########################################
    # Because we're doing an upgrade, we are NOT removing the symlinks from the
    # `/service/` directory. This allows the new version's `svscan` process to
    # start the same services which were running before.

fi

#echo "===== end %{version}-%{release} %pre $1 ====="

%post
###############################################################################
#
# post-install script
#
# We need to set up whatever "init files" are necessary to make daemontools
# start when the machine boots, and then start it running.

PATH="/usr/bin:/bin:/usr/local/bin"

#echo "===== start %{version}-%{release} %post $1 ====="
#set -x

########################################
# Check /service directory
# If it already exists, and we're doing a fresh install, warn about
# pre-existing services possibly starting on their own.

if [[ -d "/service" ]]
then
    if [[ "$1" == "1" ]]    # doing a new install, as opposed to an upgrade
    then
        if [[ -n "$( ls -d1 /service/* 2>/dev/null )" ]]
        then
            cat <<EOF

WARNING: the "/service/" directory already exists. The following pre-existing
services will start automatically when this package is finshed installing:

EOF
            ls -d1 /service/* /service/*/log | sort
            echo ''
        fi
    fi
else
    echo 'Creating /service'
    mkdir -m 0755 /service
fi

########################################
# systemd (CentOS 7, others)

if which systemctl > /dev/null 2>&1
then
    echo 'Found systemctl in PATH, assuming systemd'

    SFILE="/lib/systemd/system/daemontools.service"

    if [[ -f $SFILE ]]
    then
        echo "WARNING: renaming $SFILE to $SFILE.old"
        mv "$SFILE" "$SFILE.old"
    fi

    echo "Creating $SFILE"
    cat > $SFILE <<EOF
[Unit]
Description=daemontools
After=sysinit.target

[Service]
ExecStart=/usr/local/bin/svscanboot
Restart=always

[Install]
WantedBy=multi-user.target
EOF
    chown 0:0  $SFILE
    chmod 0644 $SFILE

    echo 'Enabling and starting daemontools'
    systemctl enable daemontools
    systemctl start daemontools

########################################
# upstart in /etc/init (CentOS 6, others)

elif [[ -d /etc/init ]]
then
    echo 'Found /etc/init, assuming upstart'

    SFILE="/etc/init/daemontools.conf"

    if [[ -f $SFILE ]]
    then
        echo "WARNING: renaming $SFILE to $SFILE.old"
        mv "$SFILE" "$SFILE.old"
    fi

    echo "Creating $SFILE"
    cat > $SFILE <<EOF
# upstart config for daemontools
start on runlevel [345]
stop on runlevel [!345]
respawn
exec /usr/local/bin/svscanboot
EOF
    chown 0:0  $SFILE
    chmod 0644 $SFILE

    echo 'Starting daemontools'
    /sbin/initctl start daemontools

########################################
# upstart in /etc/event.d (Fedora 12, others)

elif [[ -d /etc/event.d ]]
then
    echo 'Found /etc/event.d, assuming upstart'

    SFILE="/etc/event.d/daemontools.conf"

    if [[ -f $SFILE ]]
    then
        echo "WARNING: renaming $SFILE to $SFILE.old"
        mv "$SFILE" "$SFILE.old"
    fi

    echo "Creating $SFILE"
    cat > $SFILE <<EOF
# upstart config for daemontools
start on runlevel [345]
stop on runlevel [!345]
respawn
exec /usr/local/bin/svscanboot
EOF
    chown 0:0  $SFILE
    chmod 0644 $SFILE

    echo 'Starting daemontools'
    /sbin/initctl start daemontools

########################################
# inittab (most others)

elif [[ -r /etc/inittab ]]
then
    echo 'Found /etc/inittab'

    if grep svscanboot /etc/inittab >/dev/null
    then
        echo 'inittab contains an svscanboot line. I assume that svscan is already running.'
    else
        echo 'Adding svscanboot to inittab...'
        rm -f /etc/inittab'{new}'
        cp /etc/inittab /etc/inittab'{new}'
        cat >> /etc/inittab'{new}' <<EOF

SV:123456:respawn:/usr/local/bin/svscanboot
EOF
        chmod 0644 /etc/inittab'{new}'
        mv -f /etc/inittab'{new}' /etc/inittab

        kill -HUP 1
        echo 'init should start svscan now.'
    fi

########################################
# /etc/rc.local (final fallback)

elif [[ -r /etc/rc.local ]]
then
    echo 'Found /etc/rc.local'

    if grep svscanboot /etc/rc.local >/dev/null
    then
        echo 'rc.local contains an svscanboot line. I assume that svscan is already running.'
    else
        echo 'Adding svscanboot to /etc/rc.local...'
        rm -f /etc/rc.local'{new}'

        cp /etc/rc.local /etc/rc.local'{new}'
        cat >> /etc/rc.local'{new}' <<EOF

csh -cf '/usr/local/bin/svscanboot &'
EOF
        chmod 0644 /etc/rc.local'{new}'
        mv -f /etc/rc.local'{new}' /etc/rc.local

        echo 'Reboot now to start svscan.'
    fi

########################################
# final option - give up

else
    echo 'Sorry, unable to make daemontools start when the system boots.'
fi

#echo "===== end %{version}-%{release} %post $1 ====="

# end of post-install script
#
###############################################################################

%preun
###############################################################################
#
# pre-uninstall script
#
# If this is a full un-install instead of an upgrade, we need to remove
# whatever init scripts were installed by the previous package, AND shut
# down svscan and all of the services.

PATH="/usr/bin:/bin:/usr/local/bin"

#echo "===== start %{version}-%{release} %preun $1 ====="
#set -x

if [[ "$1" == "0" ]]    # doing a full un-install, as opposed to an upgrade
then
    ########################################
    # if it's in systemd, stop the service and remove the unit file

    if [[ -f /lib/systemd/system/daemontools.service ]]
    then
        echo "Found /lib/systemd/system/daemontools.service, removing from systemd"
        systemctl stop daemontools.service
        systemctl disable daemontools.service
        rm -f /lib/systemd/system/daemontools.service
    fi

    ########################################
    # if it's in upstart (either location),
    # stop it and remove the upstart config

    for DIR in /etc/init /etc/event.d
    do
        if [[ -f $DIR/daemontools.conf ]]
        then
            echo Found $DIR/daemontools.conf, removing from upstart
            /sbin/initctl stop daemontools
            rm -f $DIR/daemontools.conf
        fi
    done

    ########################################
    # if it's in inittab, remove it and send a HUP to init

    if [[ -f /etc/inittab ]]
    then
        if grep svscanboot /etc/inittab >/dev/null
        then
            echo 'Removing svscanboot from /etc/inittab'

            grep -v svscanboot /etc/inittab > /etc/inittab'{new}'
            chmod 0644 /etc/inittab'{new}'
            mv -f /etc/inittab'{new}' /etc/inittab
            kill -HUP 1
        fi
    fi

    ########################################
    # if it's in /etc/rc.local, remove it from there

    if [[ -f /etc/rc.local ]]
    then
        if grep svscanboot /etc/rc.local >/dev/null
        then
            echo 'Removing svscanboot from /etc/rc.local'

            grep -v svscanboot /etc/rc.local > /etc/rc.local'{new}'
            chmod 0644 /etc/rc.local'{new}'
            mv -f /etc/rc.local'{new}' /etc/rc.local
        fi
    fi

    ########################################
    # wherever it was, make sure it's dead

    KARG=""
    X=`ps -o pid= -C svscanboot`
    while [[ -n "$X" ]]
    do
        echo "Killing$KARG svscanboot [$X]"
        kill $KARG $X

        sleep 3
        KARG=" -9"
        X=`ps -o pid= -C svscanboot`
    done

    KARG=""
    X=`ps -o pid= -C svscan`
    while [[ -n "$X" ]]
    do
        echo "Killing$KARG svscan [$X]"
        kill $KARG $X

        sleep 3
        KARG=" -9"
        X=`ps -o pid= -C svscan`
    done

    ########################################
    # At this point, svscan is fully stopped, which means we can run `svc -dx`
    # on each service directory without having to worry about svscan starting
    # the services back up.

    svc -dx /service/* /service/*/log 2>/dev/null

    ########################################
    # Because we're doing a full un-install, we DO want to remove the symlinks
    # in the `/service/` directory, so that if daemontools is installed again
    # in the future, there won't be any surprises.

    rm -vf /service/*

fi

#echo "===== end %{version}-%{release} %preun $1 ====="

# end of pre-uninstall script
#
###############################################################################

%files
/usr/local/bin/*

%changelog
* Wed Mar 30 2016 John Simpson <jms1@voalte.com> 0.76-3
- Fixed scriptlets to properly handle the order in which `rpm` runs them
  during an upgrade (i.e. %pre ensures that svscan and all services are
  stopped when doing an upgrade, and %preun does it when doing a full
  un-install.)
- When the %preun scriptlet shuts down existing services as part of a full
  un-install, it also removes the symlinks in the `/service/` directory. This
  way if daemontools is installed again in the future, previously configured
  services won't start "on their own".
- When the %pre scriptlet shuts down existing services during an upgrade, it
  does NOT remove the symlinks in the `/service/` directory. This allows the
  new version's `svscan` process to start the same services after the
  upgrade is finished.
- If any services already exist in the `/service/` directory during a clean
  install, the RPM warns the user that they will start automatically.

* Mon Mar 07 2016 John Simpson <jms1@voalte.com> 0.76-2
- Adding systemd support for CentOS 7

* Thu Aug 29 2013 John Simpson <jms1@voalte.com> 0.76-1
- Added post-install and pre-uninstall scripts for upstart/init/rc.local

* Fri Feb 9 2007 Bruce Guenter <bruce@untroubled.org>
- Updated to version 0.76 finally.

* Thu May 5 2001 Bruce Guenter <bruceg@em.ca>
- Added a hack to allow building on glibc 2.2 systems (such as RedHat 7.x)

* Fri Dec 1 2000 Bruce Guenter <bruceg@em.ca>
- Reverted to a completely plain, unmodified install.

* Mon Mar 6 2000 Bruce Guenter <bruceg@em.ca>
- Updated to version 0.70.

* Mon Dec 6 1999 Bruce Guenter <bruceg@em.ca>
- Modified svscan init script to start all services as "down" on
  startup, and to clean up properly on exit.
- Removed /var/lock/svc directory.
